// Часы-барометр-термометр на базе 25 символьного дисплея на контроллере PCF8576. Версия 1.0. 2023 год
// автор Волков Дмитрий dimavolk@mail.ru   

#include <Wire.h>
#include <Adafruit_Sensor.h>                            // Подключаем библиотеку Adafruit_Sensor
#include <Adafruit_BMP280.h>                            // Подключаем библиотеку Adafruit_BME280
#include "GyverButton.h"
#include <EEPROM.h>

Adafruit_BMP280 bmp; // use I2C interface
Adafruit_Sensor *bmp_temp = bmp.getTemperatureSensor();
Adafruit_Sensor *bmp_pressure = bmp.getPressureSensor();

#include <microDS3231.h>
MicroDS3231 rtc;

#define ADDRESS          0x38//B0111000      //  I2C Address of PCF8576
#define DEVICE_SELECT1    B11100000    //  Device select   [Command]    [1100] 000 001 010   [A2 A1 A0]
#define DEVICE_SELECT2    B11100001    //  Device select   [Command]    [1100] 000 001 010   [A2 A1 A0]
#define DEVICE_SELECT3    B11100010    //  Device select   [Command]    [1100] 000 001 010   [A2 A1 A0]

#define MODE_SET         B11001000     //B11011001//B11001000    //  MODE SET      [C]   [10]   [LowPower]   [Enable]   [Bias]   [Mux]
#define BANK_SELECT      B11111000     //  no effect in 1:4 multiplex mode

#define BTN1 2    // пин левой кнопки
#define BTN2 3    // пин средней кнопки
#define BTN3 4    // пин правой кнопки

GButton butt1(BTN1);  // левая кнопка
GButton butt2(BTN2);  // средняя кнопка
GButton butt3(BTN3);  // правая кнопка


byte pointer_digit12[] = {0, B00000000, B00000101, B00001010, B00001111, B00010100, B00011001, B00011110, B00100011}; // указатели на адреса символов 1,2 банка (всего 8 символов в каждом банке)
byte pointer_digit3[] = {0, B00000000, B00000101, B00001010, B00001111, B00010100, B00011000, B00011100, B00100000, B00100100}; // указатели на адреса символов 3 банка (всего 9 символов в 3-м банке)
byte digit[][2] = {
      {B00001111, B00001111},  // цифра 0
      {B00010000, B10000000},  // цифра 1
      {B01001101, B00101011},  // цифра 2
      {B01001100, B00101111}, //  цифра 3  
      {B01000010, B00101100}, //  цифра 4
      {B01001110, B00100111}, //  цифра 5
      {B01001111, B00100111}, //  цифра 6
      {B00001110, B00001100}, //  цифра 7
      {B01001111, B00101111}, //  цифра 8
      {B01001110, B00101111}, //  цифра 9
      {B01000000, B00100000}, //  символ -
      {B01010000, B10100000}, //  символ +
      {B00110000, B00000000}, //  символ ,  
      {B01001000, B10001000}, //  символ градуса
      {B00001111, B00000011}, //  символ С
      {B01010001, B00100100}, //  символ м
      {B01000011, B00101100}, //  символ Н
      {B00010110, B10100001}, //  символ g  (17)
      {B00010011, B10000011}, //  символ Ц  (18)
      {B10100011, B00100000}, //  символ K  (19)
      {B01001111, B00101100}, //  символ A  (20)
      {B01001111, B00100011}, //  символ Е  (21)
      {B10000011, B01001100}, //  символ М  (22)
      {B00001111, B00001100}, //  символ П  (23)
      {B10000011, B00011100}, //  символ И  (24)
      {B11110000, B11110000}, //  символ *  (25)
      {B00000000, B00000000}, //  символ пусто  (26)
      {B00110000, B00010011}, //  символ стрелка вверх  (27)
      {B10001100, B11000000}, //  символ стрелка вниз  (28)
      {B10000000, B00010000}, //  символ /  (29)
  };

byte S_CSKA[] = {25, 18, 14, 19, 20, 10, 4, 21, 22, 23, 24, 0, 16, 25}; // Строка *ЦСКА-ЧЕМПИОН*

byte temp;
float tempReal = 0;    // переменная текущей темп-ры в градусах
float davlReal = 0;    // переменная текущего давления в мм.рт.ст.
float davlLast = 0;    // предыдущее давление
byte Hours;            // часы
byte Minutes;          // минуты
byte Seconds;          // секунды
byte t = 0;            // таймер
byte mode = 1;         // режим показа: 1 - часы, 2 - дата
int k_temp = 0;        // температурный коэф., для корректировки температуры
DateTime now;
uint32_t sec = millis() / 1000ul;      // полное количество секунд
uint32_t sec2 = millis() / 1000ul;      // полное количество секунд
uint32_t ch_sec = millis() / 1000ul;      // полное количество секунд
boolean DD =0;

void setup(){
 Wire.begin(); //conects I2C
// rtc.setTime(COMPILE_TIME);
 
//Serial.begin(9600);
// delay(100);

  butt1.setTickMode(AUTO);
  butt2.setTickMode(AUTO);
  butt3.setTickMode(AUTO);
 clear(1); clear(2); clear(3); // чистим дисплей
 CSKA(); delay(5000);
 clear(1); clear(2); clear(3); // чистим дисплей
 
 if (!bmp.begin()) {
//    Serial.println(F("Не найден датчик BMP280!"));
    while (1) delay(10);
  }
 if (EEPROM.read(100)==100) { //если это не первый запуск    
    EEPROM.get(200,k_temp);} else {EEPROM.put(200,k_temp); EEPROM.write(100,100);}     
  
 print_TD();
 print_CH();
 
 }

// Функция очистки дисплея 
void clear(byte N) {   // N - номер группы символов (банка) от 1 до 3
int adr=B00000000;

while (adr<=B00100011) {
    Wire.beginTransmission (ADDRESS) ;  
    Wire.write (MODE_SET) ;   
   switch (N) {
  case 1: 
      Wire.write (DEVICE_SELECT1);  
      break;
  case 2:
      Wire.write (DEVICE_SELECT2);  
      break;
  case 3:
    Wire.write (DEVICE_SELECT3);  
    break;
}
    Wire.write (BANK_SELECT);
    Wire.write (adr);  
    Wire.write (B00000000);
    Wire.write (B00000000);
    Wire.write (B00000000);
    Wire.endTransmission () ;
  adr=adr+B00000101;  
}
}

void print_digital(byte i, byte j, byte N, boolean z) {    // печать цифры в выбранной позиции (банк (1-3), позиция(1-8, 9-для 3-го банка), цифра или символ, запятая да/нет)

      Wire.beginTransmission (ADDRESS) ;  
      Wire.write (MODE_SET) ;   
      switch (i) {
  case 1: 
      Wire.write (DEVICE_SELECT1);  
      break;
  case 2:
      Wire.write (DEVICE_SELECT2);  
      break;
  case 3:
    Wire.write (DEVICE_SELECT3);  
    break;
}
      Wire.write (BANK_SELECT);
  if (i==3) {Wire.write (pointer_digit3[j]);} else {Wire.write (pointer_digit12[j]);}

      Wire.write (digit[N][0]);
      Wire.write (digit[N][1]);
 if (z) {Wire.write (digit[12][0]); Wire.write (digit[12][1]);}   
      Wire.endTransmission () ;
}

// Функция температуры и давления
void print_TD() {
sensors_event_t temp_event, pressure_event;  // Получаем данные о температуре и давлении
bmp_temp->getEvent(&temp_event);
bmp_pressure->getEvent(&pressure_event);
tempReal = round(temp_event.temperature*10)+k_temp; 
davlReal = round(pressure_event.pressure * 0.750064);

if (davlReal-davlLast>1) {DD=1;} else if (!davlLast==davlReal) DD=0;
davlLast=davlReal;
if (DD) { print_digital(3,9,27,0); }
  else {print_digital(3,9,28,0); }

if (tempReal<0) {
    tempReal=tempReal*(-1);
    print_digital(1,2,10,0); //печатаем "-" на экране
    } else {print_digital(1,2,11,0);} //печатаем "+" на экране

temp=tempReal/100;  // получаем цифру десятков температуры
tempReal=tempReal-temp*100;
//Serial.println(temp);
print_digital(1,3,temp,0);

temp=tempReal/10; // получаем цифру единиц температуры
tempReal=tempReal-temp*10;
print_digital(1,4,temp,1); // выводим на экран с запятой
//Serial.println(temp);

temp=tempReal; // получаем цифру десятых температуры
print_digital(1,5,temp, 0);
//Serial.println(temp);
print_digital(1,6,13,0); // печатаем на экране знак градусов и С
print_digital(1,7,14,0);


temp=davlReal/100; // получаем цифру сотен давления
davlReal=davlReal-temp*100;
//Serial.println(temp);
print_digital(3,2,temp,0);

temp=davlReal/10; // получаем цифру десятков давления
davlReal=davlReal-temp*10;
print_digital(3,3,temp,0);
//Serial.println(temp);

temp=davlReal; // получаем цифру единиц давления
print_digital(3,4,temp, 0);
//Serial.println(temp);
print_digital(3,5,15,0); // печатаем на экране ммHg
print_digital(3,6,15,0);
print_digital(3,7,16,0);
print_digital(3,8,17,0);
}

// Функция печати времени
void print_CH() {

Hours=rtc.getHours();
Minutes=rtc.getMinutes();
//Seconds=rtc.getSeconds();

temp=Hours/10; // получаем цифры десятков и едениц часов
print_digital(2,3,temp,0);
Hours=Hours-temp*10;
print_digital(2,4,Hours,0);

print_digital(2,5,10,0);

temp=Minutes/10; // получаем цифры десятков и едениц минут
print_digital(2,6,temp,0);
Minutes=Minutes-temp*10;
print_digital(2,7,Minutes,0);
}

// Функция печати даты
void print_Date() {

Hours=rtc.getDate();
Minutes=rtc.getMonth();
Seconds=rtc.getYear();

temp=Hours/10; // получаем цифры десятков и едениц числа
print_digital(2,1,temp,0);
Hours=Hours-temp*10;
print_digital(2,2,Hours,0);

print_digital(2,3,29,0);

temp=Minutes/10; // получаем цифры десятков и едениц месяца
print_digital(2,4,temp,0);
Minutes=Minutes-temp*10;
print_digital(2,5,Minutes,0);

print_digital(2,6,29,0);

Seconds=Seconds-2000;
temp=Seconds/10; // получаем цифры десятков и едениц года
print_digital(2,7,temp,0);
Seconds=Seconds-temp*10;
print_digital(2,8,Seconds,0);
}

// Функция печати ЦСКА
void CSKA() {
for (byte i=1; i <= 25; i++){
 print_digital(1, 6, i, 0); delay(30);}
//print_digital(1,6,25,0);

for (byte i=1; i <= 18; i++){
 print_digital(1, 8, i, 0); delay(30);}

//print_digital(1,8,18,0); //ЦСКА-

for (byte i=1; i <= 14; i++){
 print_digital(2, 1, i, 0); delay(30);}

//print_digital(2,1,14,0);

for (byte i=1; i <= 19; i++){
 print_digital(2, 2, i, 0); delay(30);}

//print_digital(2,2,19,0);
for (byte i=1; i <= 20; i++){
 print_digital(2, 3, i, 0); delay(30);}

//print_digital(2,3,20,0);
for (byte i=1; i <= 10; i++){
 print_digital(2, 4, i, 0); delay(30);}

//print_digital(2,4,10,0);

for (byte i=1; i <= 4; i++){
 print_digital(2, 5, i, 0); delay(30);}

//print_digital(2,5,4,0); //ЧЕМПИОН
for (byte i=1; i <= 21; i++){
 print_digital(2, 6, i, 0); delay(30);}

//print_digital(2,6,21,0);

for (byte i=1; i <= 22; i++){
 print_digital(2, 7, i, 0); delay(30);}

//print_digital(2,7,22,0);
for (byte i=1; i <= 23; i++){
 print_digital(2, 8, i, 0); delay(30);}

//print_digital(2,8,23,0);
for (byte i=1; i <= 24; i++){
 print_digital(3, 1, i, 0); delay(30);}

//print_digital(3,1,24,0);
for (byte i=1; i <= 20; i--){
 print_digital(3, 2, i, 0); delay(30);}

print_digital(3,2,0,0);
for (byte i=1; i <= 16; i++){
 print_digital(3, 3, i, 0); delay(30);}

//print_digital(3,3,16,0);
for (byte i=1; i <= 25; i++){
 print_digital(3, 5, i, 0); delay(30);}

//print_digital(3,5,25,0);
}

void loop(){
ch_sec = millis() / 1000ul;
t=ch_sec - sec;
if (mode==1) { if (t%2) {print_digital(2,5,10,0);} else {print_digital(2,5,26,0);}} // Если режим часов, то по четным мигаем

if ((t>0)and(t<49)and(mode!=1)) {mode=1; clear(2); print_CH();} else // первые 50 секунд показываем время
 {
if ((t>=49)and(t<65)and(mode!=2)) {mode=2; print_Date(); print_TD();} }  // потом показываем 15 секунд дату 
 
if (t>=65) {sec=ch_sec;}
      

if ((ch_sec - sec2) >600 ) { // Показываем строку - хранитель экрана
 clear(1); clear(2); clear(3); // чистим дисплей
 CSKA(); delay(5000); 
 clear(1); clear(2); clear(3); // чистим дисплей
 print_TD();
 if (mode==1) {print_CH();} else {print_Date();}
 sec2=ch_sec;
  }
  
  if (butt1.isClick()) { // Если нажата левая кнопка 
    now = rtc.getTime();
//  now.second = 0;
//  now.minute = 10;
//  now.hour = 50;
//  now.date = 2;
//  now.month = 9;
//  now.year = 2021;

    switch (mode) {
  case 1: // Если находимся в режиме часов
      Hours=now.hour;  // Правим часы в часах
      Hours=Hours+1;  
      if (Hours>23) {Hours=0;} 
      now.hour=Hours;
      temp=Hours/10; // получаем цифры десятков и едениц часов
      print_digital(2,3,temp,0);
      Hours=Hours-temp*10;
      print_digital(2,4,Hours,0);
      break;
  case 2: // Если находимся в режиме даты
      Hours=now.date; // Правим число в дате
      Hours=Hours+1;  
      if (Hours>31) {Hours=0;} 
      now.date=Hours;
      temp=Hours/10; // получаем цифры десятков и едениц числа
      print_digital(2,1,temp,0);
      Hours=Hours-temp*10;
      print_digital(2,2,Hours,0);
    break;
  }
rtc.setTime(now);  // загружаем в RTC
  }
  if (butt2.isClick()) { // Если нажата средняя кнопка
  now = rtc.getTime();
    switch (mode) {
  case 1: // Если находимся в режиме часов
      Minutes=now.minute; // Правим минуты в часах
      Minutes=Minutes+1;  
      if (Minutes>59) {Minutes=0;} 
      now.minute=Minutes;
      temp=Minutes/10; // получаем цифры десятков и едениц минут
      print_digital(2,6,temp,0);
      Minutes=Minutes-temp*10;
      print_digital(2,7,Minutes,0);
      break;
  case 2: // Если находимся в режиме даты
      Minutes=now.month; // Правим месяц в дате
      Minutes=Minutes+1;  
      if (Minutes>12) {Minutes=1;} 
      now.month=Minutes;
      temp=Minutes/10; // получаем цифры десятков и едениц месяца
      print_digital(2,4,temp,0);
      Minutes=Minutes-temp*10;
      print_digital(2,5,Minutes,0);
    break;
  }
  rtc.setTime(now);  // загружаем в RTC
  } 
  if (butt3.isClick()) {// Если нажата правая кнопка
  now = rtc.getTime();
    switch (mode) {
  case 1: // Если находимся в режиме часов
       k_temp=k_temp+1; // Скорректируем данные термометра
       if (k_temp>30) {k_temp=-30;}
       EEPROM.put(200,k_temp);
       print_TD();
      break;
  case 2: // Если находимся в режиме даты
      Minutes=now.year-2000; // Правим год в дате
      Minutes=Minutes+1;  
      if (Minutes>99) {Minutes=23;} 
      now.year=Minutes+2000;
      temp=Minutes/10; // получаем цифры десятков и едениц года
      print_digital(2,7,temp,0);
      Minutes=Minutes-temp*10;
      print_digital(2,8,Minutes,0);
     break;
  }
  rtc.setTime(now);  // загружаем в RTC
  } 
}
    
